@@ -7,7 +7,7 @@ This module provides some basic information about the package.
 """
 
 # Set the package release version
-version_info = (1, 0, 0)
+version_info = (1, 2, 2)
 __version__ = '.'.join(str(c) for c in version_info)
 
 # Set the package details
@@ -23,4 +23,4 @@ __about__ = ('{}\nAuthor: {} \nEmail: {} \nYear: {} \nInfo: {}'
              ''.format(__name__, __author__, __email__, __year__,
                        __description__))
 __setup_requires__ = ['pytest-runner', ]
-__tests_require__ = ['pytest', 'pytest-cov', 'pytest-pycodestyle']
+__tests_require__ = ['pytest', 'pytest-cov', 'pytest-pycodestyle', 'galsim']
