@@ -56,6 +56,9 @@ class Loc2Glob(object):
     y_npix: int
         Number of pixels on one CCD on the vertical direction.
         Default to ``4612`` (CFIS value).
+    ccd_tot: int
+        Total number of CCDs.
+        Default to ``40`` (CFIS value).
 
     Notes
     -----
@@ -81,12 +84,20 @@ class Loc2Glob(object):
         'COMMENT    __________________________'
     """
 
-    def __init__(self, x_gap=70, y_gap=425, x_npix=2048, y_npix=4612):
+    def __init__(
+        self,
+        x_gap=70,
+        y_gap=425,
+        x_npix=2048,
+        y_npix=4612,
+        ccd_tot=40
+    ):
         r"""Initialize with instrument geometry."""
         self.x_gap = x_gap
         self.y_gap = y_gap
         self.x_npix = x_npix
         self.y_npix = y_npix
+        self.ccd_tot = ccd_tot
 
     def loc2glob_img_coord(self, ccd_n, x_coor, y_coor):
         r"""Go from the local to the global img (pixel) coordinate system.
@@ -134,6 +145,20 @@ class Loc2Glob(object):
 
         return x_coor, y_coor
 
+    def x_coord_range(self):
+        r""" Return range of the x coordinate.
+        """
+        max_x = self.x_npix * 6 + self.x_gap * 5
+        min_x = self.x_npix * (-5) + self.x_gap * (-5)
+        return min_x, max_x
+
+    def y_coord_range(self):
+        r""" Return range of the y coordinate.
+        """
+        max_y = self.y_npix * 2 + self.y_gap * 1
+        min_y = self.y_npix * (-2) + self.y_gap * (-2)
+        return min_y, max_y
+
     def shift_coord(self, ccd_n):
         r"""Provide the shifting.
 
@@ -189,6 +214,137 @@ class Loc2Glob(object):
             return x_shift, y_shift
 
 
+class Loc2Glob_EUCLID_sim(object):
+    r"""Change from local to global coordinates.
+
+    Class to pass from local coordinates to global coordinates under
+    a simulation of Euclid's VIS instrument.
+    The geometrical informcation of the instrument is encoded in this
+    function.
+
+    Parameters
+    ----------
+    x_gap: int
+        Gap between the CCDs on the horizontal direction.
+    y_gap: int
+        Gap between the CCDs on the vertical direction.
+    x_npix: int (float)
+        Number of pixels on one CCD on the horizontal direction.
+    y_npix: int (float)
+        Number of pixels on one CCD on the vertical direction.
+    ccd_tot: int
+        Total number of CCDs.
+        Default to ``36`` (Euclid value).
+
+    Notes
+    -----
+    The global origin is in the middle of the focal plane. This is the
+    bottom left corner of the CCD 15.
+
+    Example of Euclid's VIS instrument geometry.
+    --------
+    >>> 'COMMENT   ------------------------',
+        'COMMENT    00  01  02  03  04  05 ',
+        'COMMENT   ------------------------',
+        'COMMENT    06  07  08  09  10  11 ',
+        'COMMENT   ------------^-----------',
+        'COMMENT    12  13  14 |15  16  17 ',
+        'COMMENT   ------------*->---------',
+        'COMMENT    18  19  20  21  22  23 ',
+        'COMMENT   ------------------------',
+        'COMMENT    24  25  26  27  28  29 ',
+        'COMMENT   ------------------------',
+        'COMMENT    30  31  32  33  34  35 ',
+        'COMMENT   ________________________'
+    """
+
+    def __init__(self, x_gap=0, y_gap=0, x_npix=4096, y_npix=4096, ccd_tot=36):
+        r"""Initialize with instrument geometry."""
+        self.x_gap = x_gap
+        self.y_gap = y_gap
+        self.x_npix = x_npix
+        self.y_npix = y_npix
+        self.ccd_tot = ccd_tot
+
+    def loc2glob_img_coord(self, ccd_n, x_coor, y_coor):
+        r"""Go from the local to the global img (pixel) coordinate system.
+
+        Global system with (0,0) in the bottom left corner of the CCD 15.
+
+        Parameters
+        ----------
+        ccd_n: int
+            CCD number of the considered positions.
+        x_coor: float
+            Local coordinate system hotizontal value.
+        y_coor: float
+            Local coordinate system vertical value.
+
+        Returns
+        -------
+        glob_x_coor: float
+            Horizontal position in global coordinate system.
+        glob_y_coor: float
+            Vertical position in global coordinate system.
+
+        """
+        # Flip axes
+        x_coor, y_coor = self.flip_coord(ccd_n, x_coor, y_coor)
+
+        # Calculate the shift
+        x_shift, y_shift = self.shift_coord(ccd_n)
+
+        # Return new coordinates
+        return x_coor + x_shift, y_coor + y_shift
+
+    def glob2loc_img_coord(self, x_coor, y_coor):
+        r""" Go from global to local coordinates.
+        """
+        # Determine the CCD
+        ccd_n = int(
+            (-y_coor // (self.y_npix + self.y_gap) + 3) * 6 + (
+                x_coor // (self.x_npix + self.x_gap)
+            ) + 3
+        )
+        # Calculate shifts
+        x_shift, y_shift = self.shift_coord(ccd_n)
+
+        return ccd_n, x_coor - x_shift, y_coor - y_shift
+
+    def flip_coord(self, ccd_n, x_coor, y_coor):
+        r"""Change of coordinate convention.
+
+        There is no fliping for Euclid.
+        """
+
+        return x_coor, y_coor
+
+    def x_coord_range(self):
+        r""" Return range of the x coordinate.
+        """
+        max_x = self.x_npix * 3 + self.x_gap * 2
+        min_x = self.x_npix * (-3) + self.x_gap * (-3)
+        return min_x, max_x
+
+    def y_coord_range(self):
+        r""" Return range of the y coordinate.
+        """
+        max_y = self.y_npix * 3 + self.y_gap * 2
+        min_y = self.y_npix * (-3) + self.y_gap * (-3)
+        return min_y, max_y
+
+    def shift_coord(self, ccd_n):
+        r"""Provide the shifting.
+
+        It is needed to go from the local coordinate
+        system origin to the global coordinate system origin.
+        """
+        x_shift = (ccd_n - 3. - (ccd_n // 6) * 6) * (self.x_gap + self.x_npix)
+        y_shift = (2. - (ccd_n // 6)) * (self.y_gap + self.y_npix)
+
+        return x_shift, y_shift
+
+
 class MccdInputs(object):
     r"""Handle inputs for the MCCD algorithm.
 
@@ -228,16 +384,29 @@ class MccdInputs(object):
         pixels to the value -1e30.
         Values beneath the threshold will be considered masked.
         Default is ``-1e5``.
+    loc2glob: object
+        The object that allows to do the coordinate conversion from local to
+        global. It is specific for each instrument's focal plane geometry.
+        If is ``None`` it defaults to the CFIS MegaCam instrument.
+    fits_tb_pos: int
+        Position in the fits file of the useful table.
+        Default is ``2`` that's the case for the CFIS data.
     """
 
     def __init__(self, separator='-', coord_x_descriptor='XWIN_IMAGE',
-                 coord_y_descriptor='YWIN_IMAGE', mask_thresh=-1e5):
+                 coord_y_descriptor='YWIN_IMAGE', mask_thresh=-1e5,
+                 loc2glob=None, fits_tb_pos=2):
         r"""Initialize class attributes."""
         self.separator = separator
         self.coord_x_descriptor = coord_x_descriptor
         self.coord_y_descriptor = coord_y_descriptor
         self.mask_thresh = mask_thresh
-        self.loc2glob = Loc2Glob()
+        if loc2glob is None:
+            self.loc2glob = Loc2Glob()
+        else:
+            self.loc2glob = loc2glob
+
+        self.fits_tb_pos = fits_tb_pos
 
         self.SNR_list = None
         self.star_list = None
@@ -438,17 +607,25 @@ class MccdInputs(object):
         DEC_list = []
 
         for it in range(number_ccd):
-            starcat = fits.open(starcat_array[it, 2])
+            starcat = fits.open(starcat_array[it, 2], memmap=False)
             ccd = starcat_array[it, 1].astype('int')
 
-            positions = np.array(
-                [self.loc2glob.loc2glob_img_coord(ccd, x, y) for x, y in
-                 zip(starcat[2].data[self.coord_x_descriptor],
-                     starcat[2].data[self.coord_y_descriptor])])
+            positions = np.array([
+                self.loc2glob.loc2glob_img_coord(ccd, x, y) for x, y in
+                zip(
+                     starcat[self.fits_tb_pos].data[self.coord_x_descriptor],
+                     starcat[self.fits_tb_pos].data[self.coord_y_descriptor]
+                )
+            ])
 
-            stars = utils.rca_format(starcat[2].data['VIGNET'])
-            masks = self.handle_mask(stars, thresh=self.mask_thresh,
-                                     apply_to_stars=True)
+            stars = utils.rca_format(
+                starcat[self.fits_tb_pos].data['VIGNET']
+            )
+            masks = self.handle_mask(
+                stars,
+                thresh=self.mask_thresh,
+                apply_to_stars=True
+            )
 
             star_list.append(stars)
             position_list.append(positions)
@@ -456,14 +633,18 @@ class MccdInputs(object):
             ccd_list.append(ccd)
 
             try:
-                SNR = starcat[2].data['SNR_WIN']
+                SNR = starcat[self.fits_tb_pos].data['SNR_WIN']
                 SNR_list.append(SNR)
             except Exception:
                 SNR_list = None
 
             try:
-                RA_list.append(starcat[2].data['XWIN_WORLD'])
-                DEC_list.append(starcat[2].data['YWIN_WORLD'])
+                RA_list.append(
+                    starcat[self.fits_tb_pos].data['XWIN_WORLD']
+                )
+                DEC_list.append(
+                    starcat[self.fits_tb_pos].data['YWIN_WORLD']
+                )
             except Exception:
                 RA_list = None
                 DEC_list = None
@@ -890,7 +1071,7 @@ def return_glob_neighbors(new_pos, obs_pos_list, val_list, n_neighbors):
     return values, positions
 
 
-def interpolation_Pi(position_list, d_comp_glob):
+def interpolation_Pi(position_list, d_comp_glob, loc2glob=None):
     r"""Create polynomial interpolation Pi matrix.
 
     Create a Pi matrix list that will be used for the interpolation
@@ -902,6 +1083,10 @@ def interpolation_Pi(position_list, d_comp_glob):
         List containing the all the positions of the training stars.
     d_comp_glob: int
         Maximum polynomial degree to be used for the Pi matrix construction.
+    loc2glob: object
+        The object that allows to do the coordinate conversion from local to
+        global. It is specific for each instrument's focal plane geometry.
+        If is ``None`` it defaults to the CFIS MegaCam instrument.
 
     Returns
     -------
@@ -910,15 +1095,17 @@ def interpolation_Pi(position_list, d_comp_glob):
         CCD (object in the position list).
 
     """
-    n_comp_glob = (d_comp_glob + 1) * (d_comp_glob + 2) // 2
+    # n_comp_glob = (d_comp_glob + 1) * (d_comp_glob + 2) // 2
 
     # Calculate max and min values of global coordinate system
     # This configuration is specific for CFIS MegaCam configuration
-    loc2glob = Loc2Glob()
-    max_x = loc2glob.x_npix * 6 + loc2glob.x_gap * 5
-    min_x = loc2glob.x_npix * (-5) + loc2glob.x_gap * (-5)
-    max_y = loc2glob.y_npix * 2 + loc2glob.y_gap * 1
-    min_y = loc2glob.y_npix * (-2) + loc2glob.y_gap * (-2)
+
+    if loc2glob is None:
+        loc2glob = Loc2Glob()
+
+    # Calculate max and min values of global coordinate system
+    min_x, max_x = loc2glob.x_coord_range()
+    min_y, max_y = loc2glob.y_coord_range()
 
     interp_Pi = [utils.poly_pos(pos=interp_pos, max_degree=d_comp_glob,
                                 center_normalice=True,
@@ -1020,95 +1207,3 @@ def get_mr_filters(data_shape, opt, n_scales=3, coarse=False, trim=False):
         return filters
     else:
         return filters[:-1]
-
-
-class MomentInterpolator(object):
-    r"""Allow to interpolate moments from a bin image.
-
-    Bin image like the one from the MeanShapes function.
-
-    Notes
-    -----
-    Not used for the moment.
-    """
-
-    def __init__(self, moment_map, n_neighbors=1000,
-                 rbf_function='thin_plate'):
-        r"""Initialize class attributes."""
-        # Save variables
-        self.loc2glob = Loc2Glob()
-        self.n_neighbors = n_neighbors
-        self.rbf_function = rbf_function
-        self.moment_map = np.zeros(moment_map.shape)
-        self.x_pix = self.loc2glob.x_npix
-        self.y_pix = self.loc2glob.y_npix
-
-        # Define parameters
-        self.n_ccd = moment_map.shape[0]  # 40
-        self.x_grid = moment_map.shape[1]  # 20
-        self.y_grid = moment_map.shape[2]  # 40
-        self.bin_x = self.x_pix / self.x_grid
-        self.bin_y = self.y_pix / self.y_grid
-
-        # Correct MegaCam origin conventions
-        for ccd_it in range(self.n_ccd):
-            for it_x in range(self.x_grid):
-                for it_y in range(self.y_grid):
-
-                    if ccd_it < 18 or ccd_it in [36, 37]:
-                        # swap x axis so origin is on top-right
-                        x = it_x
-                        y = it_y
-
-                    else:
-                        # swap y axis so origin is on bottom-left
-                        x = self.x_grid - it_x - 1
-                        y = self.y_grid - it_y - 1
-
-                    self.moment_map[ccd_it, x, y] = moment_map[ccd_it, it_x,
-                                                               it_y]
-
-        # Generate local generic grid
-        x_lin = np.linspace(start=self.bin_x / 2,
-                            stop=self.x_pix - self.bin_x / 2, num=self.x_grid)
-        y_lin = np.linspace(start=self.bin_y / 2,
-                            stop=self.y_pix - self.bin_y / 2, num=self.y_grid)
-        xv, yv = np.meshgrid(x_lin, y_lin, indexing='ij')
-        self.xv = xv
-        self.yv = yv
-
-        # Generate global positions for the bins
-        self.x_pos = np.zeros(moment_map.shape)
-        self.y_pos = np.zeros(moment_map.shape)
-        for ccd_it in range(self.n_ccd):
-            x_glob, y_glob = self.loc2glob.loc2glob_img_coord(
-                ccd_n=ccd_it,
-                x_coor=np.copy(self.xv.flatten()),
-                y_coor=np.copy(self.yv.flatten()))
-
-            self.x_pos[ccd_it, :, :] = x_glob.reshape(self.x_grid,
-                                                      self.y_grid)
-            self.y_pos[ccd_it, :, :] = y_glob.reshape(self.x_grid,
-                                                      self.y_grid)
-
-    def interpolate_position(self, target_x, target_y):
-        r"""Interpolate positions."""
-        # Calculate distances
-        res_x = self.x_pos.flatten() - target_x
-        res_y = self.y_pos.flatten() - target_y
-        dist = np.sqrt(res_x ** 2 + res_y ** 2)
-
-        # Select bins to use. The n_neighbors closest positions
-        sort_idxs = np.argsort(dist)[:self.n_neighbors]
-
-        # Extract values
-        x_pos_interp = self.x_pos.flatten()[sort_idxs]
-        y_pos_interp = self.y_pos.flatten()[sort_idxs]
-        val_interp = self.moment_map.flatten()[sort_idxs]
-
-        # Generate the interpolation function
-        rbf = Rbf(x_pos_interp, y_pos_interp,
-                  val_interp, function=self.rbf_function)
-        output_val = rbf(target_x, target_y)
-
-        return output_val
